PROFANITY_LIST = {
    # Common swear words
    "fuck", "fucking", "fucked", "fucker", "motherfucker", "fucks", "fuk", "fck", "f u c k",
    "shit", "shitty", "shite", "bullshit", "horseshit", "sh1t", "sh1te",
    "ass", "asshole", "arse", "arsehole", "asshat", "asswipe",
    "bitch", "bitches", "b1tch", "b1tches",
    "dick", "d1ck", "dickhead", "dickweed",
    "cunt", "cunts", "c*nt", "cuntface",
    "cock", "c0ck", "cocksucker", "cockhead",
    "piss", "pissed", "pissing", "piss off",
    "bastard", "bastards",
    "wanker", "wank", "w4nk", "wanker",
    "twat", "twats",
    "prick", "pricks",

    # Slurs and hate speech (use with extreme caution and context)
    "nigger", "n1gger", "niggah", "nigga",
    "faggot", "fag", "f4g", "faggots",
    "retard", "retarded", "tard",
    "chink", "gook", "spic", "wetback", "kike", "raghead", "sandnigger",

    # Sexual/explicit terms
    "tits", "titties", "boobs", "boobies",
    "pussy", "pussies",
    "clit", "clitoris",
    "blowjob", "bj", "suck dick", "deepthroat",
    "cum", "cumming", "jizz", "spunk",
    "porn", "porno", "xxx",

    # Combinations and evasions
    "f@ck", "f**k", "fu*k", "f.u.c.k", "phuck", "fuhk",
    "sh!t", "s h i t", "sh*t",
    "b!tch", "b*tch",
    "c*nt", "cuntz",
    "a55", "a$$", "a55hole",

    # Miscellaneous offensive
    "whore", "slut", "skank", "hoe", "ho",
    "douche", "douchebag",
    "crap", "crappy",
    "damn", "goddamn", "goddammit", "god damn",
    "hell", "hel", "h3ll",
    "jesus christ", "jesus fucking christ",
}

# Case-insensitive matching
PROFANITY_SET = {word.lower() for word in PROFANITY_LIST}


import unittest
from username_safety.checker import UsernameChecker

class TestUsernameChecker(unittest.TestCase):
    def setUp(self):
        self.checker = UsernameChecker()

    def test_length(self):
        self.assertTrue(self.checker.check_length("valid")[0])
        self.assertFalse(self.checker.check_length("no")[0])
        self.assertFalse(self.checker.check_length("thisiswaytoolongforausernamewhichshouldfail")[0])

    def test_unsafe_chars(self):
        self.assertTrue(self.checker.check_unsafe_chars("valid_user-123")[0])
        self.assertFalse(self.checker.check_unsafe_chars("invalid user")[0])
        self.assertFalse(self.checker.check_unsafe_chars("user@name")[0])

    def test_profanity(self):
        self.assertTrue(self.checker.check_profanity("cleanuser")[0])
        self.assertFalse(self.checker.check_profanity("shit123")[0])

    def test_similarity(self):
        self.assertTrue(self.checker.check_similarity("different", "John Doe")[0])
        self.assertFalse(self.checker.check_similarity("johndoe", "John Doe")[0])

    def test_validate(self):
        valid, _ = self.checker.validate("gooduser")
        self.assertTrue(valid)

        valid, errors = self.checker.validate("badword", real_name="Bad Word")
        self.assertFalse(valid)
        self.assertTrue(len(errors) > 0)

if __name__ == '__main__':
    unittest.main()



import re
import difflib
from .profanity_list import PROFANITY_LIST

class UsernameChecker:
    def __init__(self):
        self.profanity_list = PROFANITY_LIST

    def check_length(self, username, min_len=3, max_len=20):
        if len(username) < min_len:
            return False, f"Username too short (min {min_len})"
        if len(username) > max_len:
            return False, f"Username too long (max {max_len})"
        return True, "OK"

    def check_unsafe_chars(self, username, allowed_regex=r'^[a-zA-Z0-9_-]+$'):
        if not re.match(allowed_regex, username):
            return False, "Username contains unsafe characters"
        return True, "OK"

    def check_profanity(self, username):
        # Basic check: exact match or substring match
        # For a more robust check, we might want to normalize the username
        username_lower = username.lower()
        for word in self.profanity_list:
            if word in username_lower:
                return False, "Username contains profanity"
        return True, "OK"

    def check_similarity(self, username, real_name, threshold=0.8):
        if not real_name:
            return True, "OK"
        
        # Simple similarity check using SequenceMatcher
        ratio = difflib.SequenceMatcher(None, username.lower(), real_name.lower()).ratio()
        if ratio >= threshold:
            return False, f"Username is too similar to real name (similarity: {ratio:.2f})"
        return True, "OK"

    def validate(self, username, real_name=None):
        checks = [
            self.check_length(username),
            self.check_unsafe_chars(username),
            self.check_profanity(username),
            self.check_similarity(username, real_name)
        ]

        errors = []
        for success, message in checks:
            if not success:
                errors.append(message)

        if errors:
            return False, errors
        return True, ["OK"]
